#include <basedef.h>
#include <vmm.h>
#include <debug.h>
#include <vxdwraps.h>
#include <vwin32.h>
#include <configmg.h>
     
char *  __cdecl strcpy(char *, const char *); 
char *  __cdecl strcmp(char *, const char *); 
int  __cdecl strlen(const char *);
#pragma intrinsic(strcpy)
#pragma intrinsic(strlen)
#pragma intrinsic(strcmp)
     
#ifdef DEBUG
#define DPRINTF0(fmt) _Sprintf(dbgbuf, fmt ); \
     
Out_Debug_String( dbgbuf )
#define DPRINTF1(fmt,a1)_Sprintf(dbgbuf, fmt, a1 ); \
     
Out_Debug_String( dbgbuf )
#define DPRINTF2(fmt,a1,a2) _Sprintf(dbgbuf, fmt, a1, a2 ); \
     
Out_Debug_String( dbgbuf )
#else
#define DPRINTF0(fmt)
#define DPRINTF1(fmt,arg1)
#define DPRINTF2(fmt,arg1,arg2)
#endif
     
char dbgbuf[120];
char matchid[100];
char class[20];
char subclass[20];
     
DWORD OnW32Deviceiocontrol(PDIOCPARAMETERS p); 
BOOL  OnSysDynamicDeviceInit();
BOOL  OnSysDynamicDeviceExit();
char * strstr( const char *str1, const char *str2 ); 
BOOL MatchById(DEVNODE node, char *matchid );
BOOL MatchByClass(DEVNODE node, char *class );
BOOL MatchBySubClass( DEVNODE node, char *subclass ); 
void VisitNode( DEVNODE node );
VOID TraverseHardwareTree(DEVNODE node );
     
BOOL OnSysDynamicDeviceInit(void)
{
   strcpy(matchid, "PCI\\");
   strcpy(class, "Display");
   strcpy(subclass, "joystick");
   TraverseHardwareTree( NULL );                 
   return TRUE;
}
     
BOOL OnSysDynamicDeviceExit(void)
{
   return TRUE;
}
     
char * __cdecl strstr (const char * str1, const char * str2) 
{
   char *cp = (char *) str1;
   char *s1, *s2;
     
   if ( !*str2 )
      return((char *)str1);
     
   while (*cp)
   {
      s1 = cp;
      s2 = (char *) str2;
     
      while ( *s1 && *s2 && !(*s1-*s2) )
         s1++, s2++;
     
      if (!*s2)
         return(cp);
     
      cp++;
   }
     
   return(NULL);
     
}
     
BOOL MatchById( DEVNODE node, char *matchid ) 
{
   char devid[100];
     
   CM_Get_Device_ID( node, devid, sizeof(devid), 0 ); 
   if (strstr(devid, matchid))
      return TRUE;               
}
     
BOOL MatchByClass( DEVNODE node, char *class ) 
{
   char           regdata[100];
   unsigned long  size;
   CONFIGRET      rc;
     
   size = sizeof(regdata);
   rc = CM_Read_Registry_Value( node, NULL, "Class", 
                           REG_SZ, regdata, &size, 
                           CM_REGISTRY_HARDWARE );
   if (rc == CR_SUCCESS)
   {
      if (strcmp(regdata, class) == 0)
         return TRUE;               
   }
   return FALSE;
}
     
BOOL MatchBySubClass( DEVNODE node, char *subclass ) 
{
   char           regdata[100];
   unsigned long  size;
   CONFIGRET      rc;
     
   size = sizeof(regdata);
   rc = CM_Read_Registry_Value( node, "Drivers", "SubClasses", 
                           REG_SZ, regdata, &size, 
                           CM_REGISTRY_SOFTWARE );
   if (rc == CR_SUCCESS)
   {
      if (strstr(regdata, subclass))
         return TRUE;               
   }
   return FALSE;
}
     
void VisitNode(DEVNODE node )
{
// char devid[100];
// CM_Get_Device_ID( node, devid, sizeof(devid), 0 ); 
//   DPRINTF2("node=%x id=%s\n", node, devid );
     
   if (MatchById(node, matchid))
   {
      DPRINTF0("NODE MATCH ON ID\n");
   }
   if (MatchByClass(node, class ))
   {
      DPRINTF0("NODE MATCH ON CLASS\n");
   }
   if (MatchBySubClass(node, subclass )) 
   {
      DPRINTF0("NODE MATCH ON SUBCLASS\n");
   }
}
     
     
VOID TraverseHardwareTree(DEVNODE node) 
{
   DEVNODE child, sibling;
     
   if (!node)
   {
      CM_Locate_DevNode(&node, NULL, 0);
      if (node == 0)
         return;
   }
     
   VisitNode(node);
     
   if (CM_Get_Child(&child, node, 0) != CR_SUCCESS)
      return;
     
   else
   {
      TraverseHardwareTree(child);
     
      while (CM_Get_Sibling(&sibling, child, 0) == CR_SUCCESS) 
      {
         TraverseHardwareTree(sibling);
         child = sibling;        
      }
   }
}
     
     
